/* The Game of Life
   Copyright Chris Broadribb 2002

   You may use and distribute this program for free, but you may not
   modify it in any way except for modifying life.dat to use a
   different starting cell configuration.

   This program requires the file life.dat.  If you choose to write
   some of the cell configurations to disk, it will write them to
   cells1.dat, cells2.dat etc.

   I've included some sample life.dat files (life.bk1, life.bk2,
   life.bk3, life.bk4).
*/

#include <stdio.h>

int iscellalive(char cells[20][20], int column, int row);
void readcells(char cells[20][20] );
void displaycells(char cells[20][20]);
void writecells(char cells[20][20]);
void changecells(char cells[20][20]);
int countneighbours(char cells[20][20], int column, int row);
int getposition(int position);
void swapimage(char cells[20][20], char newcells[20][20]);
void userinput(char *choice);

void main(void)
{
	char cells[20][20];
	int quit;
	char choice;

	quit = 0;
	clrscr();
	puts("      The Game Of Life");
	puts("Copyright Chris Broadribb 2002");
	readcells(cells);
	displaycells(cells);
	while(!quit)
	{
		userinput(&choice);
		if(choice == 'd')
		{
			changecells(cells);
			displaycells(cells);
		}
		else if(choice == 'w')
			writecells(cells);
		else if(choice == 'q')
			quit = 1;
	}
}


int iscellalive(char cells[20][20], int column, int row)
{
	if(cells[column][row] == '*')
		return 1;
	else
		return 0;
}


void readcells(char cells[20][20])
{
	int column, row;
	FILE *fp;
	char temp[50];

	fp = fopen("life.dat","rt");
	if(!fp)
	{
		puts("Error, can't find life.dat");
		exit(1);
	}
	for(column=0;column<20;column++)
	{
		strcpy(temp,"");
		if(fgets(temp,48,fp) == NULL)
		{
			puts("The format of life.dat is incorrect.");
			fclose(fp);
			exit(1);
		}
		sscanf(temp,"%c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c %c",
		&cells[column][0], &cells[column][1], &cells[column][2], &cells[column][3], &cells[column][4],
		&cells[column][5], &cells[column][6], &cells[column][7], &cells[column][8], &cells[column][9],
		&cells[column][10], &cells[column][11], &cells[column][12], &cells[column][13], &cells[column][14],
		&cells[column][15], &cells[column][16], &cells[column][17], &cells[column][18], &cells[column][19]);
	}
	fclose(fp);
}


void displaycells(char cells[20][20])
{
	int column, row;

	clrscr();
	puts("*** The Game Of Life ***  Copyright Chris Broadribb 2002");
	for(column=0;column<20;column++)
	{
		for(row=0;row<20;row++)
		{
			if(cells[column][row] == '*')
				printf("*");
			else
				printf(" ");
		}
		printf("\n"); // new line
	}
}


void writecells(char cells[20][20])
{
	int column, row;
	FILE *fp;
	char filename[13];
	static int pattern = 0;

	pattern ++;
	sprintf(filename,"cells%i.dat",pattern);
	fp = fopen(filename,"wt");
	for(column=0;column<20;column++)
	{
		for(row=0;row<20;row++)
			fprintf(fp,"%c ",cells[column][row]);
		fprintf(fp,"\n");
	}
	fclose(fp);
	printf("\nThe pattern has been written to %s.\n",filename);
}


void changecells(char cells[20][20])
{
	char newcells[20][20];
	int column, row, alive, neighbours;

	for(column=0;column<20;column++)
		for(row=0;row<20;row++)
			newcells[column][row] = '-';
	for(column=0;column<20;column++)
		for(row=0;row<20;row++)
		{
			alive = iscellalive(cells,column,row);
			neighbours = countneighbours(cells,column,row);
			if(alive)
			{
				if(neighbours == 2 || neighbours == 3)
				{
					newcells[column][row] = '*';
				}
			}
			else if(neighbours == 3)
				newcells[column][row] = '*';
		}
	swapimage(cells,newcells);
}


int countneighbours(char cells[20][20], int column, int row)
{
	int xpos, ypos, neighbours;

	neighbours = 0;
	ypos = row;
	xpos = getposition(column+1);
	neighbours += iscellalive(cells,xpos,ypos);
	xpos = getposition(column-1);
	neighbours += iscellalive(cells,xpos,ypos);
	xpos = column;
	ypos = getposition(row+1);
	neighbours += iscellalive(cells,xpos,ypos);
	ypos = getposition(row-1);
	neighbours += iscellalive(cells,xpos,ypos);
	xpos = getposition(column+1);
	neighbours += iscellalive(cells,xpos,ypos);
	ypos = getposition(row+1);
	neighbours += iscellalive(cells,xpos,ypos);
	xpos = getposition(column-1);
	neighbours += iscellalive(cells,xpos,ypos);
	ypos = getposition(row-1);
	neighbours += iscellalive(cells,xpos,ypos);
	return neighbours;
}


int getposition(int position)
{
	if(position > 19)
		return 0;
	else if (position < 0)
		return 19;
	else
		return position;
}


void swapimage(char cells[20][20], char newcells[20][20])
{
	int column, row;

	for(column=0;column<20;column++)
		for(row=0;row<20;row++)
			cells[column][row] = newcells[column][row];
}


void userinput(char *choice)
{
	puts("Enter d to display the next iteration of the cells, w to write the");
	puts("current pattern to disk, or q to quit the program.");
	do
	{
		printf(">  ");
		*choice = getche();
	} while(*choice != 'd' && *choice != 'w' && *choice != 'q');
}
